@@ -34,7 +34,11 @@ import Control.Monad (when, unless)
 import Control.Monad.Trans.Resource (ResourceT, runResourceT)
 import Control.Monad.State.Strict
 import Control.Monad.Reader
+#if MIN_VERSION_aeson(2,1,2)
+import Data.Aeson hiding (AesonException)
+#else
 import Data.Aeson
+#endif
 #if MIN_VERSION_aeson(2,0,0)
 import qualified Data.Aeson.Key as K
 import qualified Data.Aeson.KeyMap as M
@@ -42,8 +46,12 @@ import Data.Aeson.KeyMap (KeyMap)
 #else
 import qualified Data.HashMap.Strict as M
 #endif
-import Data.Aeson.Internal (JSONPath, JSONPathElement(..), formatError)
+#if MIN_VERSION_aeson(2,2,0)
+import Data.Aeson.Types hiding (AesonException, parse)
+#else
 import Data.Aeson.Types hiding (parse)
+import Data.Aeson.Internal (JSONPath, JSONPathElement(..), formatError)
+#endif
 import qualified Data.Attoparsec.Text as Atto
 import Data.Bits (shiftL, (.|.))
 import Data.ByteString (ByteString)
@@ -51,7 +59,7 @@ import qualified Data.ByteString.Builder as BB
 import qualified Data.ByteString.Lazy as BL
 import Data.ByteString.Builder.Scientific (scientificBuilder)
 import Data.Char (toUpper, ord)
-import Data.List
+import qualified Data.List as List
 import Data.Conduit ((.|), ConduitM, runConduit)
 import qualified Data.Conduit.List as CL
 import qualified Data.HashSet as HashSet
@@ -314,14 +322,14 @@ parseM mergedKeys a front = do
               if s == "<<"
                          then case o of
                                   Object l  -> return (merge l)
-                                  Array l -> return $ merge $ foldl' mergeObjects M.empty $ V.toList l
+                                  Array l -> return $ merge $ List.foldl' mergeObjects M.empty $ V.toList l
                                   _          -> al
                          else al
             parseM mergedKeys' a al'
     where mergeObjects al (Object om) = M.union al om
           mergeObjects al _           = al
 
-          merge xs = (Set.fromList (M.keys xs \\ M.keys front), M.union front xs)
+          merge xs = (Set.fromList (M.keys xs List.\\ M.keys front), M.union front xs)
 
 parseSrc :: ReaderT JSONPath (ConduitM Event Void Parse) val
          -> ConduitM () Event Parse ()
